import asyncio
import aioredis
import logging
import os

logger = logging.getLogger(__name__)


class Redis(object):
    """
    ENV requires:
    REDIS_ADDRESS=redis://localhost
    REDIS_DB=
    REDIS_PASSWORD=
    """

    def __init__(self):
        logger.debug('Redis init')
        self.loop = getattr(self, "loop", asyncio.get_event_loop())
        self.redis = self.loop.run_until_complete(self.__redis_init())
        super(Redis, self).__init__()

    def __del__(self):
        self.redis.close()
        self.loop.run_until_complete(self.redis.wait_closed())
        deleter = getattr(super(Redis, self), "__del__", None)
        if callable(deleter):
            deleter()

    async def __redis_init(self):
        db = os.getenv('REDIS_DB', None)
        password = os.getenv('REDIS_PASSWORD', None)
        connect = {
            "address": os.getenv('REDIS_ADDRESS', 'redis://localhost'),
            "db": int(db) if db is not None else None,
            "password": password if password is not None and len(password) else None,
        }
        return await aioredis.create_redis_pool(**connect, loop=self.loop)

    async def lock(self, key, expire=30):
        if await self.redis.execute('SETNX', key, 1):
            await self.redis.execute('EXPIRE', key, int(expire))
            logger.info('Locked: `{}`'.format(key))
            return True
        else:
            logger.info('Already Locked: `{}`'.format(key))
            return False



import asyncio
import logging
import os

logger = logging.getLogger(__name__)


class Service(object):

    def __init__(self, worker, loop=None):
        logging.basicConfig(
            level=logging.getLevelName(os.getenv('LOG_LEVEL', 'INFO')),
            format="%(asctime)-15s [{}] %(levelname)8s %(name)20s %(message)s".format(worker)
        )
        logger.debug('Service init')

        if loop is None:
            self.loop = getattr(self, "loop", asyncio.get_event_loop())
        else:
            self.loop = loop
        super(Service, self).__init__()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass



import asyncio
import aioamqp
import logging
import json

import os

logger = logging.getLogger(__name__)


class RMQ(object):
    """
    ENV requires:
    RMQ_HOST=127.0.0.1
    RMQ_PORT=5672
    RMQ_USER=guest
    RMQ_PASS=guest
    RMQ_EXCHANGE=
    RMQ_EXCHANGE_TYPE=direct
    """

    def __init__(self):
        logger.debug('RMQ init')
        self.loop = getattr(self, "loop", asyncio.get_event_loop())
        self.exchange = os.getenv('RMQ_EXCHANGE', '')
        self.transport, self.protocol, self.channel = self.loop.run_until_complete(self.__rmq_init())
        super(RMQ, self).__init__()

    def __del__(self):
        self.loop.run_until_complete(self.__rmq_close())
        deleter = getattr(super(RMQ, self), "__del__", None)
        if callable(deleter):
            deleter()

    async def __rmq_close(self):
        await self.channel.close()
        await self.protocol.close()
        self.transport.close()

    async def __rmq_init(self):
        rmq = {
            "host": os.getenv('RMQ_HOST', '127.0.0.1'),
            "port": os.getenv('RMQ_PORT', 5672),
            "login": os.getenv('RMQ_USER', 'guest'),
            "password": os.getenv('RMQ_PASS', 'guest'),
        }
        transport, protocol = await aioamqp.connect(**rmq)
        logger.info('RMQ connected')
        channel = await protocol.channel()

        if self.exchange != '':
            type_name = os.getenv('RMQ_EXCHANGE_TYPE', 'direct')
            await channel.exchange_declare(exchange_name=self.exchange, type_name=type_name)

        return transport, protocol, channel

    async def queue_declare(self, queues):
        for queue in queues:
            await self.channel.queue_declare(**queue)

    async def publish(self, data, queue, headers=None):
        await self.channel.basic_publish(
            payload=json.dumps(data),
            exchange_name=self.exchange,
            routing_key=queue,
            properties={
                'headers': headers if headers else {}
            }
        )


import asyncio
import aiopg
import logging
import os

logger = logging.getLogger(__name__)


class Postgres(object):
    """
    ENV requires:
    PG_HOST=127.0.0.1
    PG_PORT=5432
    PG_USER=guest
    PG_PASS=guest
    PG_DB=postgres
    """

    def __init__(self):
        logger.debug('Postgres init')
        self.loop = getattr(self, "loop", asyncio.get_event_loop())
        self.pool = self.loop.run_until_complete(self.__db_init())
        super(Postgres, self).__init__()

    def __del__(self):
        self.pool.close()
        deleter = getattr(super(Postgres, self), "__del__", None)
        if callable(deleter):
            deleter()

    async def __db_init(self):
        dsn = 'dbname={} user={} password={} host={} port={}'.format(
            os.getenv('PG_DB', 'postgres'),
            os.getenv('PG_USER', 'postgres'),
            os.getenv('PG_PASS', 'postgres'),
            os.getenv('PG_HOST', '127.0.0.1'),
            os.getenv('PG_PORT', 5432),
        )
        pool = await aiopg.create_pool(dsn, timeout=1, echo=os.getenv('LOG_LEVEL', None) == 'DEBUG')
        logger.info('Postgres connected')
        async with pool.acquire() as conn:
            async with conn.cursor() as cursor:
                await self.create_tables(cursor)
        return pool

    async def create_tables(self, cursor):
        pass



from setuptools import setup
import src

setup(
    name='mcsrv',
    version=src.__version__,
    install_requires=[
        "aioredis",
        "aioamqp",
        "aiopg",
    ],
    description="Package for small service start",
    author='Stepan Pyzhov',
    maintainer='Stepan Pyzhov',
    packages=['mcsrv'],
    package_dir={'mcsrv': 'src'},
)
